//
//  PPHybridViewController.m
//

#import "PPHybridViewController.h"

@interface PPHybridViewController(/* Private */)

@property (nonatomic, strong) NSURL *requestURL;

@end

@implementation PPHybridViewController

CGFloat topHeight = 40.0f;
CGFloat bottomHeight = 60.0f;

#if ! __has_feature(objc_arc)
- (void)dealloc {
    
    [super dealloc];
}
#endif

- (void) loadView {
	[super loadView];
    
    // NavigationBar 는 사용하지 않으므로 Hidden
    self.navigationController.navigationBarHidden = YES;
    
    //safeArea 영역에 대한 색상
    [self.view setBackgroundColor:[UIColor blueColor]];
    
    //background 영역에 대한 색상
    [self.backgroundView setBackgroundColor:[UIColor grayColor]];
    
    //webView 영역에 대한 색상
    [self.poperaWebview setBackgroundColor:[UIColor orangeColor]];
    
    [self addViewCustomSize];
}

// backgroundView에 SafeArea를 적용할지에 대한 플래그
-(BOOL)useSafeArea
{
    return YES;
}

-(void)addViewCustomSize
{
    //상단 네이티브 뷰
    UIView *topView = [[UIView alloc] init];
    topView.tag = 100;
    [self.backgroundView addSubview:topView];
    [topView setFrame:CGRectMake(0, 0, self.backgroundView.frame.size.width, topHeight)];
    [topView setAutoresizingMask:(UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleBottomMargin)];
    [topView setBackgroundColor:[UIColor redColor]];
    
    //하단 네이티브 뷰
    UIView *bottomView = [[UIView alloc] init];
    [self.backgroundView addSubview:bottomView];
    [bottomView setBackgroundColor:[UIColor greenColor]];
    [bottomView setFrame:CGRectMake(0, self.backgroundView.frame.size.height - bottomHeight, self.backgroundView.frame.size.width, bottomHeight)];
    [bottomView setAutoresizingMask:(UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin)];
    
    //웹뷰 사이즈 조절
    [self.poperaWebview setFrame:CGRectMake(0, topHeight, self.backgroundView.frame.size.width, self.backgroundView.frame.size.height - topHeight - bottomHeight)];
}

// 홈버튼이 있는 단말의 경우 getPoperaWebViewFrame을 사용할지에 대한 플래그값
-(BOOL)useCustomPoperaFrame
{
    return YES;
}

// 홈버튼이 있는 단말의 경우 useStatusBarView 여부에 따라 20픽셀에 대한 처리가 있어 구현이 필요함
-(CGRect)getPoperaWebViewFrame
{
    // 홈버튼 있는 경우 적용
    int status = 20;
    if([self useStatusBarView])
    {
        status = 20;
        UIView *topView = [self.backgroundView viewWithTag:100];
        [topView setFrame:CGRectMake(0, status, self.backgroundView.frame.size.width, topHeight)];
    }
    else
    {
        status = 0;
    }
    return CGRectMake(0, status + topHeight, self.backgroundView.frame.size.width, self.backgroundView.frame.size.height - status - bottomHeight - topHeight);
}

// 홈버튼이 있는 단말에서 상단 statusBar 영역을 비울지에 대한 플래그값
-(BOOL)useStatusBarView
{
    return YES;
}

/*
 * @brief StatusBar Style 변경
 */
- (UIStatusBarStyle)preferredStatusBarStyle {
    //self.backgroundView.backgroundColor = [UIColor blackColor];
    return UIStatusBarStyleLightContent;
}

/*
 * @brief StatusBar 표시 여부
 */
- (BOOL)prefersStatusBarHidden {
    return NO;
}


/*
 * @brief 현재화면으로 시작되거나 되돌아오고 나면 호출
 */
- (void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];
}

/*
 * @brief 현재화면에서 벗어나기 직전에 호출
 */
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
 
    // 화면 이동전 현재화면에 떠있는 Modal 창 닫기
    if ( self.presentedViewController != nil ) {
        [self.presentedViewController dismissViewControllerAnimated:animated completion:^{
            
        }];
    }
}

/*
 * @brief 이동될 페이지경로 저장
 */
-(void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler
{
    _requestURL = navigationAction.request.URL;
    [super webView:webView decidePolicyForNavigationAction:navigationAction decisionHandler:decisionHandler];
}

/*
 * @brief 외부 페이지가 열린 상태에서 로딩 오류가 발생했을 경우에 대한 처리.
 */
-(void)webView:(WKWebView *)webView didFailNavigation:(WKNavigation *)navigation withError:(NSError *)error
{
    [super webView:webView didFailNavigation:navigation withError:error];
}

/*
 * @brief 부모 클래스에서 처리되지 않은 기능을 확장영역에서 처리할 수 있도록 추가함.
 */
- (void) exWebView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler
{
    if (![_requestURL.scheme hasPrefix:@"http"] && [[UIApplication sharedApplication] canOpenURL:_requestURL]) {
        //  HTTP가 아닌 일반 사용자 정의 scheme의 경우 해당 Skip 처리하도록 추가함.
        
        [[UIApplication sharedApplication] openURL:_requestURL];
        decisionHandler(WKNavigationActionPolicyCancel); // 내부적으로 처리함.
        return;
    }
    else {
        // AppFree 사이트의 경우 http://itunes.apple.com 를 open시도 후 바로 itms-services:// 스키마를 날리고 있음.
        // 예제(http://itunes.apple.com/us/app/aebpeuli-touchen-appfree/id506292960?mt=8)처럼 명시적으로 appStore로 이동하려는 경우
        
        if ([_requestURL.absoluteString hasPrefix:@"http://itunes.apple.com/"]) {
            [[UIApplication sharedApplication] openURL:_requestURL];
            decisionHandler(WKNavigationActionPolicyCancel); // 내부적으로 처리함.
            return;
        }
    }
    
    // 부모에서 처리하도록 함.
    decisionHandler(WKNavigationActionPolicyAllow);
}


@synthesize requestURL = _requestURL;

@end
